@@ -3,7 +3,7 @@
 #
 # Interface to Clipper C++ code
 #
-#  $Revision: 1.13 $ $Date: 2016/03/24 00:52:57 $
+#  $Revision: 1.15 $ $Date: 2018/06/14 02:02:36 $
 #
 
 validxy <- function(P) {
@@ -52,9 +52,9 @@ polysimplify <-
     A <- ensuredouble(A)
     storage.mode(pft) <- "integer"
     storage.mode(x0) <- storage.mode(y0) <- storage.mode(eps) <- "double"
-    result <- .Call("Csimplify",
+    result <- .Call(eCsimplify,
                     A, pft, x0, y0, eps,
-                    PACKAGE = "polyclip")
+                    PACKAGE="polyclip")
     return(aspolygonlist(result))
   }
 
@@ -64,7 +64,8 @@ polyclip <-
            ...,
            eps, x0, y0,
            fillA=c("evenodd", "nonzero", "positive", "negative"),
-           fillB=c("evenodd", "nonzero", "positive", "negative")
+           fillB=c("evenodd", "nonzero", "positive", "negative"),
+           closed=TRUE
            ) {
     # validate parameters and convert to integer codes
     op <- match.arg(op)
@@ -82,6 +83,7 @@ polyclip <-
       if(validxy(B)) B <- list(B) else
       stop("Argument B should be a list of lists, each containing vectors x,y")
     }
+    # 
     # determine value of 'eps' if missing
     if(missing(eps) || missing(x0) || missing(y0)) {
       xr <- range(range(unlist(lapply(A, xrange))),
@@ -97,10 +99,11 @@ polyclip <-
     B <- ensuredouble(B)
     storage.mode(ct) <- storage.mode(pftA) <- storage.mode(pftB) <- "integer"
     storage.mode(x0) <- storage.mode(y0) <- storage.mode(eps) <- "double"
-    ans <- .Call("Cclipbool",
+    storage.mode(closed) <- "logical"
+    ans <- .Call(eCclipbool,
                  A, B, pftA, pftB, ct,
-                 x0, y0, eps,
-                 PACKAGE = "polyclip")
+                 x0, y0, eps, closed,
+                 PACKAGE="polyclip")
     return(aspolygonlist(ans))
   }
 
@@ -135,9 +138,10 @@ polyoffset <-
     storage.mode(delta) <-
       storage.mode(miterlim) <- storage.mode(arctol) <- "double"
     storage.mode(x0) <- storage.mode(y0) <- storage.mode(eps) <- "double"
-    ans <- .Call("Cpolyoffset", A, delta, jt,
+    ans <- .Call(eCpolyoffset,
+                 A, delta, jt,
                  miterlim, arctol, x0, y0, eps,
-                 PACKAGE = "polyclip")
+                 PACKAGE="polyclip")
     return(aspolygonlist(ans))
   }
 
@@ -184,9 +188,10 @@ polylineoffset <-
     storage.mode(delta) <- storage.mode(miterlim) <-
       storage.mode(arctol) <- "double"
     storage.mode(x0) <- storage.mode(y0) <- storage.mode(eps) <- "double"
-    ans <- .Call("Clineoffset", A, delta, jt, et,
+    ans <- .Call(eClineoffset,
+                 A, delta, jt, et,
                  miterlim, arctol, x0, y0, eps,
-                 PACKAGE = "polyclip")
+                 PACKAGE="polyclip")
     return(aspolygonlist(ans))
   }
 
@@ -224,8 +229,34 @@ polyminkowski <-
     B <- ensuredouble(B)
     storage.mode(x0) <- storage.mode(y0) <- storage.mode(eps) <- "double"
     storage.mode(closed) <- "logical"
-    result <- .Call("Cminksum",
+    result <- .Call(eCminksum,
                     A, B, closed, x0, y0, eps,
-                    PACKAGE = "polyclip")
+                    PACKAGE="polyclip")
     return(aspolygonlist(result))
   }
+
+pointinpolygon <-
+  function(P, A, eps, x0, y0) {
+    # validate arguments
+    if(!validxy(P))
+      stop("Argument P should be a list containing vectors x,y")
+    if(!validxy(A))
+      stop("Argument A should be a list containing vectors x,y")
+    # determine value of 'eps' if missing
+    if(missing(eps) || missing(x0) || missing(y0)) {
+      xr <- xrange(A)
+      yr <- yrange(A)
+      if(missing(eps)) eps <- max(diff(xr), diff(yr))/1e9
+      if(missing(x0)) x0 <- mean(xr)
+      if(missing(y0)) y0 <- mean(yr)
+    } 
+    # call clipper library
+    A <- ensurexydouble(A)
+    P <- ensurexydouble(P)
+    storage.mode(x0) <- storage.mode(y0) <- storage.mode(eps) <- "double"
+    ans <- .Call(eCpiptest,
+                 P, A,
+                 x0, y0, eps,
+                 PACKAGE="polyclip")
+    return(ans)
+  }
